@@ -27,6 +27,8 @@
  * \brief vectorコンテナ
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-11-29
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.4.0/cstl/vector.h $
+ * $Id: vector.h 181 2007-12-11 00:48:23Z katono $
  */
 #ifndef CSTL_VECTOR_H_INCLUDED
 #define CSTL_VECTOR_H_INCLUDED
@@ -49,6 +51,22 @@
 #define CSTL_VECTOR_MAGIC(x)
 #endif
 
+#ifndef CSTL_ALGORITHM_INTERFACE
+#define CSTL_ALGORITHM_INTERFACE(Name, Type)
+#endif
+
+#ifndef CSTL_ALGORITHM_IMPLEMENT
+#define CSTL_ALGORITHM_IMPLEMENT(Name, Type, DIRECT_ACCESS)
+#endif
+
+
+#define CSTL_VECTOR_AT(self, idx)	(self)->buf[(idx)]
+#define CSTL_VECTOR_SIZE(self)		(self)->end
+#define CSTL_VECTOR_EMPTY(self)		((self)->end == 0)
+#define CSTL_VECTOR_CAPACITY(self)	(self)->nelems
+#define CSTL_VECTOR_FULL(self)		(CSTL_VECTOR_SIZE((self)) == CSTL_VECTOR_CAPACITY((self)))
+#define CSTL_VECTOR_CLEAR(self)		(self)->end = 0
+
 
 /*! 
  * \brief インターフェイスマクロ
@@ -62,7 +80,6 @@ typedef struct Name##_t Name;\
 CSTL_VECTOR_BEGIN_EXTERN_C()\
 Name *Name##_new(size_t n);\
 void Name##_delete(Name *self);\
-int Name##_assign(Name *self, Name *x, size_t idx, size_t n);\
 int Name##_push_back(Name *self, Type elem);\
 Type Name##_pop_back(Name *self);\
 size_t Name##_size(Name *self);\
@@ -76,19 +93,21 @@ Type *Name##_at(Name *self, size_t idx);\
 Type Name##_front(Name *self);\
 Type Name##_back(Name *self);\
 int Name##_insert(Name *self, size_t idx, Type elem);\
-int Name##_insert_array(Name *self, size_t idx, Type *elems, size_t n);\
+int Name##_insert_n(Name *self, size_t idx, size_t n, Type elem);\
+int Name##_insert_array(Name *self, size_t idx, const Type *elems, size_t n);\
+int Name##_insert_range(Name *self, size_t idx, Name *x, size_t xidx, size_t n);\
 void Name##_erase(Name *self, size_t idx, size_t n);\
 void Name##_swap(Name *self, Name *x);\
+CSTL_ALGORITHM_INTERFACE(Name, Type)\
 CSTL_VECTOR_END_EXTERN_C()\
 
 
-/*! 
- * \brief 実装マクロ
- * 
- * \param Name コンテナ名
- * \param Type 要素の型
- */
-#define CSTL_VECTOR_IMPLEMENT(Name, Type)	\
+
+#define CSTL_VECTOR_IMPLEMENT_BASE(Name, Type)	\
+static int Name##_expand(Name *self, size_t size);\
+static void Name##_move_forward(Name *self, size_t first, size_t last, size_t n);\
+static void Name##_move_backward(Name *self, size_t first, size_t last, size_t n);\
+static int Name##_insert_n_no_elem(Name *self, size_t idx, size_t n);\
 /*! 
  * \brief vector構造体
  */\
@@ -99,18 +118,6 @@ struct Name##_t {\
 	CSTL_VECTOR_MAGIC(Name *magic;)\
 };\
 \
-static int Name##_full(Name *self)\
-{\
-	return (Name##_size(self) == Name##_capacity(self));\
-}\
-\
-static int Name##_expand(Name *self, size_t size)\
-{\
-	size_t n;\
-	n = (size > Name##_capacity(self) * 2) ? size : Name##_capacity(self) * 2;\
-	return Name##_reserve(self, n);\
-}\
-\
 Name *Name##_new(size_t n)\
 {\
 	Name *self;\
@@ -126,7 +133,7 @@ Name *Name##_new(size_t n)\
 	}\
 	self->end = 0;\
 	self->buf = buf;\
-	CSTL_VECTOR_MAGIC(self->magic = self;)\
+	CSTL_VECTOR_MAGIC(self->magic = self);\
 	return self;\
 }\
 \
@@ -134,81 +141,79 @@ void Name##_delete(Name *self)\
 {\
 	assert(self && "Vector_delete");\
 	assert(self->magic == self && "Vector_delete");\
-	CSTL_VECTOR_MAGIC(self->magic = 0;)\
+	CSTL_VECTOR_MAGIC(self->magic = 0);\
 	free(self->buf);\
 	free(self);\
 }\
 \
-int Name##_assign(Name *self, Name *x, size_t idx, size_t n)\
-{\
-	size_t i;\
-	assert(self && "Vector_assign");\
-	assert(self->magic == self && "Vector_assign");\
-	assert(x && "Vector_assign");\
-	assert(x->magic == x && "Vector_assign");\
-	assert(Name##_size(x) >= idx + n && "Vector_assign");\
-	if (self == x) {\
-		Name##_erase(self, idx + n, Name##_size(self) - (idx + n));\
-		Name##_erase(self, 0, idx);\
-	} else {\
-		if (n > Name##_capacity(self)) {\
-			if (!Name##_expand(self, Name##_capacity(self) + n)) return 0;\
-		}\
-		Name##_clear(self);\
-		for (i = 0; i < n; i++) {\
-			Name##_push_back(self, *Name##_at(x, i));\
-		}\
-	}\
-	return 1;\
-}\
-\
+
+
+#define CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name, Type)	\
 int Name##_push_back(Name *self, Type elem)\
 {\
 	assert(self && "Vector_push_back");\
 	assert(self->magic == self && "Vector_push_back");\
-	if (Name##_full(self)) {\
-		if (!Name##_expand(self, Name##_capacity(self) + 1)) return 0;\
-	}\
+	if (!Name##_expand(self, CSTL_VECTOR_SIZE(self) + 1)) return 0;\
 	self->buf[self->end] = elem;\
 	self->end++;\
 	return 1;\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_POP_BACK(Name, Type)	\
 Type Name##_pop_back(Name *self)\
 {\
 	assert(self && "Vector_pop_back");\
 	assert(self->magic == self && "Vector_pop_back");\
-	assert(!Name##_empty(self) && "Vector_pop_back");\
+	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_pop_back");\
 	self->end--;\
 	return self->buf[self->end];\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_SIZE(Name, Type)	\
 size_t Name##_size(Name *self)\
 {\
 	assert(self && "Vector_size");\
 	assert(self->magic == self && "Vector_size");\
-	return self->end;\
+	return CSTL_VECTOR_SIZE(self);\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_CAPACITY(Name, Type)	\
 size_t Name##_capacity(Name *self)\
 {\
 	assert(self && "Vector_capacity");\
 	assert(self->magic == self && "Vector_capacity");\
-	return self->nelems;\
+	return CSTL_VECTOR_CAPACITY(self);\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_EMPTY(Name, Type)	\
 int Name##_empty(Name *self)\
 {\
 	assert(self && "Vector_empty");\
 	assert(self->magic == self && "Vector_empty");\
-	return (self->end == 0);\
+	return CSTL_VECTOR_EMPTY(self);\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_CLEAR(Name, Type)	\
 void Name##_clear(Name *self)\
 {\
 	assert(self && "Vector_clear");\
 	assert(self->magic == self && "Vector_clear");\
-	self->end = 0;\
+	CSTL_VECTOR_CLEAR(self);\
+}\
+\
+
+#define CSTL_VECTOR_IMPLEMENT_RESERVE(Name, Type)	\
+static int Name##_expand(Name *self, size_t size)\
+{\
+	size_t n;\
+	if (size <= CSTL_VECTOR_CAPACITY(self)) return 1;\
+	n = (size > CSTL_VECTOR_CAPACITY(self) * 2) ? size : CSTL_VECTOR_CAPACITY(self) * 2;\
+	return Name##_reserve(self, n);\
 }\
 \
 int Name##_reserve(Name *self, size_t n)\
@@ -216,7 +221,7 @@ int Name##_reserve(Name *self, size_t n)\
 	Type *newbuf;\
 	assert(self && "Vector_reserve");\
 	assert(self->magic == self && "Vector_reserve");\
-	if (n <= Name##_capacity(self)) return 1;\
+	if (n <= CSTL_VECTOR_CAPACITY(self)) return 1;\
 	newbuf = (Type *) realloc(self->buf, sizeof(Type) * n);\
 	if (!newbuf) return 0;\
 	self->buf = newbuf;\
@@ -224,14 +229,16 @@ int Name##_reserve(Name *self, size_t n)\
 	return 1;\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_SHRINK(Name, Type)	\
 void Name##_shrink(Name *self, size_t n)\
 {\
 	Type *newbuf;\
 	assert(self && "Vector_shrink");\
 	assert(self->magic == self && "Vector_shrink");\
-	if (n >= Name##_capacity(self)) return;\
-	if (n < Name##_size(self)) {\
-		n = Name##_size(self);\
+	if (n >= CSTL_VECTOR_CAPACITY(self)) return;\
+	if (n < CSTL_VECTOR_SIZE(self)) {\
+		n = CSTL_VECTOR_SIZE(self);\
 	}\
 	self->nelems = n;\
 	if (!n) n = 1;\
@@ -241,101 +248,187 @@ void Name##_shrink(Name *self, size_t n)\
 	}\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_RESIZE(Name, Type)	\
 int Name##_resize(Name *self, size_t n, Type elem)\
 {\
 	size_t size;\
 	assert(self && "Vector_resize");\
 	assert(self->magic == self && "Vector_resize");\
-	size = Name##_size(self);\
+	size = CSTL_VECTOR_SIZE(self);\
 	if (size >= n) {\
 		self->end = n;\
 	} else {\
-		size_t i;\
-		if (!Name##_expand(self, n)) {\
+		register size_t i;\
+		if (!Name##_insert_n_no_elem(self, size, n - size)) {\
 			return 0;\
 		}\
-		for (i = 0; i < n - size; i++) {\
-			Name##_push_back(self, elem);\
+		for (i = size; i < n; i++) {\
+			CSTL_VECTOR_AT(self, i) = elem;\
 		}\
 	}\
 	return 1;\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_AT(Name, Type)	\
 Type *Name##_at(Name *self, size_t idx)\
 {\
 	assert(self && "Vector_at");\
 	assert(self->magic == self && "Vector_at");\
-	assert(Name##_size(self) > idx && "Vector_at");\
-	return &self->buf[idx];\
+	assert(CSTL_VECTOR_SIZE(self) > idx && "Vector_at");\
+	return &CSTL_VECTOR_AT(self, idx);\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_FRONT(Name, Type)	\
 Type Name##_front(Name *self)\
 {\
 	assert(self && "Vector_front");\
 	assert(self->magic == self && "Vector_front");\
-	assert(!Name##_empty(self) && "Vector_front");\
+	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_front");\
 	return self->buf[0];\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_BACK(Name, Type)	\
 Type Name##_back(Name *self)\
 {\
 	assert(self && "Vector_back");\
 	assert(self->magic == self && "Vector_back");\
-	assert(!Name##_empty(self) && "Vector_back");\
+	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_back");\
 	return self->buf[self->end - 1];\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name, Type)	\
 static void Name##_move_forward(Name *self, size_t first, size_t last, size_t n)\
 {\
-	size_t i;\
+	register size_t i;\
 	for (i = last; i > first; i--) {\
 		self->buf[i - 1 + n] = self->buf[i - 1];\
 	}\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name, Type)	\
 static void Name##_move_backward(Name *self, size_t first, size_t last, size_t n)\
 {\
-	size_t i;\
+	register size_t i;\
 	for (i = first; i < last; i++) {\
 		self->buf[i - n] = self->buf[i];\
 	}\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_INSERT_N_NO_ELEM(Name, Type)	\
+static int Name##_insert_n_no_elem(Name *self, size_t idx, size_t n)\
+{\
+	if (!Name##_expand(self, CSTL_VECTOR_SIZE(self) + n)) return 0;\
+	Name##_move_forward(self, idx, self->end, n);\
+	self->end += n;\
+	return 1;\
+}\
+\
+
+#define CSTL_VECTOR_IMPLEMENT_INSERT(Name, Type)	\
+CSTL_VECTOR_IMPLEMENT_INSERT_N_NO_ELEM(Name, Type)\
+\
 int Name##_insert(Name *self, size_t idx, Type elem)\
 {\
 	assert(self && "Vector_insert");\
 	assert(self->magic == self && "Vector_insert");\
-	assert(Name##_size(self) >= idx && "Vector_insert");\
-	return Name##_insert_array(self, idx, &elem, 1);\
+	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert");\
+	return Name##_insert_array(self, idx, (const Type *) &elem, 1);\
 }\
 \
-int Name##_insert_array(Name *self, size_t idx, Type *elems, size_t n)\
+int Name##_insert_n(Name *self, size_t idx, size_t n, Type elem)\
 {\
-	size_t i, j;\
-	assert(self && "Vector_insert_array");\
-	assert(self->magic == self && "Vector_insert_array");\
-	assert(Name##_size(self) >= idx && "Vector_insert_array");\
-	assert(elems && "Vector_insert_array");\
-	if (Name##_size(self) + n > Name##_capacity(self)) {\
-		if (!Name##_expand(self, Name##_capacity(self) + n)) return 0;\
+	register size_t i;\
+	assert(self && "Vector_insert_n");\
+	assert(self->magic == self && "Vector_insert_n");\
+	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_n");\
+	if (!Name##_insert_n_no_elem(self, idx, n)) {\
+		return 0;\
 	}\
-	Name##_move_forward(self, idx, self->end, n);\
-	self->end += n;\
-	for (i = idx, j = 0; j < n; i++, j++) {\
-		self->buf[i] = elems[j];\
+	for (i = 0; i < n; i++) {\
+		self->buf[idx + i] = elem;\
 	}\
 	return 1;\
 }\
 \
+int Name##_insert_array(Name *self, size_t idx, const Type *elems, size_t n)\
+{\
+	register size_t i;\
+	assert(self && "Vector_insert_array");\
+	assert(self->magic == self && "Vector_insert_array");\
+	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_array");\
+	assert(elems && "Vector_insert_array");\
+	if (!Name##_insert_n_no_elem(self, idx, n)) {\
+		return 0;\
+	}\
+	for (i = 0; i < n; i++) {\
+		self->buf[idx + i] = ((Type *) elems)[i];\
+	}\
+	return 1;\
+}\
+\
+
+#define CSTL_VECTOR_IMPLEMENT_INSERT_RANGE(Name, Type)	\
+int Name##_insert_range(Name *self, size_t idx, Name *x, size_t xidx, size_t n)\
+{\
+	register size_t i;\
+	assert(self && "Vector_insert_range");\
+	assert(self->magic == self && "Vector_insert_range");\
+	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_range");\
+	assert(x && "Vector_insert_range");\
+	assert(x->magic == x && "Vector_insert_range");\
+	assert(CSTL_VECTOR_SIZE(x) >= xidx + n && "Vector_insert_range");\
+	assert(CSTL_VECTOR_SIZE(x) >= n && "Vector_insert_range");\
+	assert(CSTL_VECTOR_SIZE(x) > xidx && "Vector_insert_range");\
+	if (!Name##_insert_n_no_elem(self, idx, n)) {\
+		return 0;\
+	}\
+	if (self == x) {\
+		if (idx <= xidx) {\
+			for (i = 0; i < n; i++) {\
+				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + n + i);\
+			}\
+		} else if (xidx < idx && idx < xidx + n) {\
+			for (i = 0; i < idx - xidx; i++) {\
+				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + i);\
+			}\
+			for (i = 0; i < n - (idx - xidx); i++) {\
+				CSTL_VECTOR_AT(self, idx + (idx - xidx) + i) = CSTL_VECTOR_AT(self, idx + n + i);\
+			}\
+		} else {\
+			for (i = 0; i < n; i++) {\
+				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + i);\
+			}\
+		}\
+	} else {\
+		for (i = 0; i < n; i++) {\
+			CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(x, xidx + i);\
+		}\
+	}\
+	return 1;\
+}\
+\
+
+#define CSTL_VECTOR_IMPLEMENT_ERASE(Name, Type)	\
 void Name##_erase(Name *self, size_t idx, size_t n)\
 {\
 	assert(self && "Vector_erase");\
 	assert(self->magic == self && "Vector_erase");\
-	assert(Name##_size(self) >= idx + n && "Vector_erase");\
+	assert(CSTL_VECTOR_SIZE(self) >= idx + n && "Vector_erase");\
+	assert(CSTL_VECTOR_SIZE(self) >= n && "Vector_erase");\
+	assert(CSTL_VECTOR_SIZE(self) > idx && "Vector_erase");\
 	Name##_move_backward(self, idx + n, self->end, n);\
 	self->end -= n;\
 }\
 \
+
+#define CSTL_VECTOR_IMPLEMENT_SWAP(Name, Type)	\
 void Name##_swap(Name *self, Name *x)\
 {\
 	size_t tmp_end;\
@@ -357,4 +450,34 @@ void Name##_swap(Name *self, Name *x)\
 }\
 \
 
+
+/*! 
+ * \brief 実装マクロ
+ * 
+ * \param Name コンテナ名
+ * \param Type 要素の型
+ */
+#define CSTL_VECTOR_IMPLEMENT(Name, Type)	\
+CSTL_VECTOR_IMPLEMENT_BASE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_POP_BACK(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_SIZE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_CAPACITY(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_EMPTY(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_CLEAR(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_RESERVE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_SHRINK(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_RESIZE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_AT(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_FRONT(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_BACK(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_INSERT(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_INSERT_RANGE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_ERASE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_SWAP(Name, Type)\
+CSTL_ALGORITHM_IMPLEMENT(Name, Type, CSTL_VECTOR_AT)\
+
+
 #endif /* CSTL_VECTOR_H_INCLUDED */
