@@ -4,8 +4,8 @@
 #include <stdio.h>
 #include <cstl/string.h>
 
-STRING_INTERFACE(String, char)
-STRING_IMPLEMENT(String, char)
+CSTL_STRING_INTERFACE(String, char)
+CSTL_STRING_IMPLEMENT(String, char)
 
 /* 標準入力から個々の単語を抽出し、それぞれの文字を逆順に出力する */
 int main(int argc, char *argv[])
@@ -14,20 +14,20 @@ int main(int argc, char *argv[])
 	String *line = String_new(1024);
 	/* 区切り文字 */
 	String *delims = String_new(8);
-	String_assign(delims, " \t,.;", NPOS);
+	String_assign(delims, " \t,.;", CSTL_NPOS);
 
 	while (fgets(buf, 1024, stdin)) {
 		size_t begIdx, endIdx;
-		String_assign(line, buf, NPOS);
-		String_erase(line, String_length(line)-1, NPOS); /* '\n'を取り除く */
+		String_assign(line, buf, CSTL_NPOS);
+		String_erase(line, String_length(line)-1, CSTL_NPOS); /* '\n'を取り除く */
 		/* 最初の単語の先頭を検索する */
-		begIdx = String_find_first_not_of(String_c_str(line), String_c_str(delims), 0, NPOS);
+		begIdx = String_find_first_not_of(String_c_str(line), String_c_str(delims), 0, CSTL_NPOS);
 		/* 単語の先頭が見つかるまで繰り返す */
-		while (begIdx != NPOS) {
+		while (begIdx != CSTL_NPOS) {
 			size_t i;
 			/* 単語の末尾を検索する */
-			endIdx = String_find_first_of(String_c_str(line), String_c_str(delims), begIdx, NPOS);
-			if (endIdx == NPOS) {
+			endIdx = String_find_first_of(String_c_str(line), String_c_str(delims), begIdx, CSTL_NPOS);
+			if (endIdx == CSTL_NPOS) {
 				/* 単語の終わりは行末である */
 				endIdx = String_length(line);
 			}
@@ -37,7 +37,7 @@ int main(int argc, char *argv[])
 			}
 			putchar(' ');
 			/* 次の単語の先頭を検索する */
-			begIdx = String_find_first_not_of(String_c_str(line), String_c_str(delims), endIdx, NPOS);
+			begIdx = String_find_first_not_of(String_c_str(line), String_c_str(delims), endIdx, CSTL_NPOS);
 		}
 		putchar('\n');
 	}
