@@ -1,14 +1,44 @@
-#include "test.h"
-
+#include <stdio.h>
+#include <string.h>
+#include <assert.h>
 #include <time.h>
+#include "../cstl/ring.h"
+#include "../cstl/map.h"
+#include "rbtree_debug.h"
+#include "heap.h"
+#ifdef MY_MALLOC
+double buf[1024*1024/sizeof(double)];
+Heap heap;
+#define malloc(s)		Heap_alloc(&heap, s)
+#define realloc(p, s)	Heap_realloc(&heap, p, s)
+#define free(p)			Heap_free(&heap, p)
+#endif
+
+/* ring */
+CSTL_RING_INTERFACE(IntRing, int)
+CSTL_RING_IMPLEMENT(IntRing, int)
+
+
+/* map */
+#ifdef CSTLGEN
+#include "IntIntMapA.h"
+#include "IntIntMMapA.h"
+#else
+
+CSTL_MAP_INTERFACE(IntIntMapA, int, int)
+CSTL_MAP_DEBUG_INTERFACE(IntIntMapA)
+
+CSTL_MULTIMAP_INTERFACE(IntIntMMapA, int, int)
+CSTL_MAP_DEBUG_INTERFACE(IntIntMMapA)
 
 /* int */
-MAP_IMPLEMENT(IntIntMapA, int, int, SIMPLE_CMP, ASC)
-MAP_DEBUG_IMPLEMENT(IntIntMapA, int, int, SIMPLE_CMP, ASC, %d, %d, VISUAL)
-static IntIntMapA *ia;
+CSTL_MAP_IMPLEMENT(IntIntMapA, int, int, CSTL_LESS)
+CSTL_MAP_DEBUG_IMPLEMENT(IntIntMapA, int, int, CSTL_LESS, %d, %d, VISUAL)
 
-MULTIMAP_IMPLEMENT(IntIntMMapA, int, int, SIMPLE_CMP, ASC)
-MAP_DEBUG_IMPLEMENT(IntIntMMapA, int, int, SIMPLE_CMP, ASC, %d, %d, VISUAL)
+CSTL_MULTIMAP_IMPLEMENT(IntIntMMapA, int, int, CSTL_LESS)
+CSTL_MAP_DEBUG_IMPLEMENT(IntIntMMapA, int, int, CSTL_LESS, %d, %d, VISUAL)
+#endif
+static IntIntMapA *ia;
 static IntIntMMapA *ima;
 
 
@@ -24,17 +54,17 @@ void map_init_hoge(void)
 {
 	int i;
 	int tmp;
-	IntDeque *q = IntDeque_new(SIZE/2);
+	IntRing *q = IntRing_new(SIZE/2);
 	srand(time(0));
 /*    printf("hoge data\n");*/
 	for (i = 0; i < SIZE/2; i++) {
-		IntDeque_push_back(q, i);
+		IntRing_push_back(q, i);
 	}
 	for (i = 0; i < SIZE; i++) {
-		if (!IntDeque_empty(q)) {
-			int rnd = rand() % IntDeque_size(q);
-			tmp = *IntDeque_at(q, rnd);
-			IntDeque_erase(q, rnd, 1);
+		if (!IntRing_empty(q)) {
+			int rnd = rand() % IntRing_size(q);
+			tmp = *IntRing_at(q, rnd);
+			IntRing_erase(q, rnd, 1);
 		} else {
 			tmp = i/2;
 		}
@@ -46,7 +76,8 @@ void map_init_hoge(void)
 /*        printf("%4d: int[%3d], double[%5g], ptr[%p], str[%s]\n",*/
 /*                i, hoge_int[i], hoge_double[i], hoge_ptr[i], hoge_str[i]);*/
 	}
-	IntDeque_delete(q);
+	HEAP_DUMP_OVERFLOW(&heap);
+	IntRing_delete(q);
 }
 
 void MapTest_test_1_1(void)
@@ -195,14 +226,21 @@ void MapTest_test_1_1(void)
 /*    IntIntMapA_print(x);*/
 	assert(IntIntMapA_size(ia) == 3);
 	assert(IntIntMapA_size(x) == 5);
-	/* assign */
-	assert(IntIntMapA_assign(x, IntIntMapA_begin(ia), IntIntMapA_end(ia)));
-	assert(IntIntMapA_size(ia) == IntIntMapA_size(x));
+	/* insert_range */
+/*    IntIntMapA_print(ia);*/
+/*    IntIntMapA_print(x);*/
+	assert(IntIntMapA_insert_range(x, IntIntMapA_begin(ia), IntIntMapA_end(ia)));
+/*    IntIntMapA_print(ia);*/
+/*    IntIntMapA_print(x);*/
+	assert(IntIntMapA_size(x) == 8);
+	assert(IntIntMapA_insert_range(x, IntIntMapA_begin(ia), IntIntMapA_end(ia)));
+	assert(IntIntMapA_size(x) == 8);
 	IntIntMapA_print(ia);
 	IntIntMapA_print(x);
 
 
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	IntIntMapA_delete(ia);
 	IntIntMapA_delete(x);
 }
@@ -326,6 +364,7 @@ void MapTest_test_1_2(void)
 	assert(IntIntMMapA_size(ima) == count);
 	printf("count: %d\n", count);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	IntIntMMapA_delete(ima);
 }
 
@@ -348,3 +387,15 @@ void MapTest_run(void)
 	MapTest_test_1_2();
 }
 
+
+int main(void)
+{
+#ifdef MY_MALLOC
+	Heap_init(&heap, buf, sizeof buf, sizeof buf[0]);
+#endif
+	MapTest_run();
+#ifdef MY_MALLOC
+	HEAP_DUMP_LEAK(&heap, 0);
+#endif
+	return 0;
+}
