@@ -27,6 +27,8 @@
  * \brief set/mapデバッグ用
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-11-14
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.4.0/test/rbtree_debug.h $
+ * $Id: rbtree_debug.h 181 2007-12-11 00:48:23Z katono $
  *
  * 赤黒木の表示とベリファイ
  */
@@ -75,16 +77,16 @@ static void Name##_p(Name##RBTreeNode *self, size_t depth)\
 	} else {\
 		if (map) {\
 			printf("n[%p], p[%p], l[%p], r[%p], k["#format1"], v["#format2"], c[%s], d[%d]",\
-				self, self->parent, self->left, self->right, self->key, self->value,\
+				(void *) self, (void *) self->parent, (void *) self->left, (void *) self->right, self->key, self->value,\
 				Name##_str_color(self->color), depth);\
 		} else {\
 			printf("n[%p], p[%p], l[%p], r[%p], v["#format1"], c[%s], d[%d]",\
-				self, self->parent, self->left, self->right, self->key,\
+				(void *) self, (void *) self->parent, (void *) self->left, (void *) self->right, self->key,\
 				Name##_str_color(self->color), depth);\
 		}\
-		if (Name##RBTreeNode_is_root(self)) {\
+		if (CSTL_RBTREE_NODE_IS_ROOT(self)) {\
 			printf(", root");\
-		} else if (Name##RBTreeNode_is_nil(self->left) && Name##RBTreeNode_is_nil(self->right)) {\
+		} else if (CSTL_RBTREE_NODE_IS_NIL(self->left, Name##RBTree) && CSTL_RBTREE_NODE_IS_NIL(self->right, Name##RBTree)) {\
 			printf(", leaf");\
 		}\
 	}\
@@ -94,12 +96,12 @@ static void Name##_p(Name##RBTreeNode *self, size_t depth)\
 static void Name##Node_print(Name##RBTreeNode *self)\
 {\
 	static size_t depth = 0;\
-	if (Name##RBTreeNode_is_nil(self)) {\
+	if (CSTL_RBTREE_NODE_IS_NIL(self, Name##RBTree)) {\
 		return;\
 	}\
 	depth++;\
 	if (Name##_max_depth < depth) Name##_max_depth = depth;\
-	if (Name##RBTreeNode_is_nil(self->right) && Name##RBTreeNode_is_nil(self->left)) {\
+	if (CSTL_RBTREE_NODE_IS_NIL(self->right, Name##RBTree) && CSTL_RBTREE_NODE_IS_NIL(self->left, Name##RBTree)) {\
 		if (Name##_min_depth > depth) Name##_min_depth = depth;\
 	}\
 	if (visual) {\
@@ -117,7 +119,7 @@ static void Name##Node_print(Name##RBTreeNode *self)\
 void Name##_print(Name *self)\
 {\
 	Name##RBTreeNode *root;\
-	assert(Name##RBTreeNode_is_head(self->tree));\
+	assert(CSTL_RBTREE_NODE_IS_HEAD(self->tree));\
 	root = Name##RBTree_get_root(self->tree);\
 	Name##_max_depth = 0;\
 	Name##_min_depth = UINT_MAX;\
@@ -129,7 +131,7 @@ void Name##_print(Name *self)\
 static size_t Name##RBTreeNode_black_count(Name##RBTreeNode *t, Name##RBTreeNode *root)\
 {\
 	size_t count;\
-	if (Name##RBTreeNode_is_nil(t)) {\
+	if (CSTL_RBTREE_NODE_IS_NIL(t, Name##RBTree)) {\
 		return 0;\
 	}\
 	count = t->color == CSTL_RBTREE_BLACK ? 1 : 0;\
@@ -149,25 +151,27 @@ int Name##_verify(Name *self)\
 	Name##RBTree *tree = self->tree;\
 	if (Name##RBTree_empty(tree) || Name##RBTree_begin(tree) == Name##RBTree_end(tree)) {\
 		return Name##RBTree_empty(tree) && Name##RBTree_begin(tree) == Name##RBTree_end(tree) &&\
-			tree->left == &Name##RBTree_nil && tree->right == &Name##RBTree_nil && tree->parent == &Name##RBTree_nil;\
+			tree->left == (Name##RBTreeNode *) &Name##RBTree_nil &&\
+			tree->right == (Name##RBTreeNode *) &Name##RBTree_nil &&\
+			tree->parent == (Name##RBTreeNode *) &Name##RBTree_nil;\
 	}\
 	len = Name##RBTreeNode_black_count(Name##RBTree_begin(tree), Name##RBTree_get_root(tree));\
 	for (pos = Name##RBTree_begin(tree); pos != Name##RBTree_end(tree); pos = Name##RBTree_next(pos)) {\
 		l = pos->left;\
 		r = pos->right;\
 		if (pos->color == CSTL_RBTREE_RED) {\
-			if ((!Name##RBTreeNode_is_nil(l) && l->color == CSTL_RBTREE_RED) ||\
-				(!Name##RBTreeNode_is_nil(r) && r->color == CSTL_RBTREE_RED)) {\
+			if ((!CSTL_RBTREE_NODE_IS_NIL(l, Name##RBTree) && l->color == CSTL_RBTREE_RED) ||\
+				(!CSTL_RBTREE_NODE_IS_NIL(r, Name##RBTree) && r->color == CSTL_RBTREE_RED)) {\
 				return 0;\
 			}\
 		}\
-		if (!Name##RBTreeNode_is_nil(l) && Compare(pos->key, l->key) < 0) {\
+		if (!CSTL_RBTREE_NODE_IS_NIL(l, Name##RBTree) && Compare(pos->key, l->key) < 0) {\
 			return 0;\
 		}\
-		if (!Name##RBTreeNode_is_nil(r) && Compare(r->key, pos->key) < 0) {\
+		if (!CSTL_RBTREE_NODE_IS_NIL(r, Name##RBTree) && Compare(r->key, pos->key) < 0) {\
 			return 0;\
 		}\
-		if (Name##RBTreeNode_is_nil(l) && Name##RBTreeNode_is_nil(r) &&\
+		if (CSTL_RBTREE_NODE_IS_NIL(l, Name##RBTree) && CSTL_RBTREE_NODE_IS_NIL(r, Name##RBTree) &&\
 			Name##RBTreeNode_black_count(pos, Name##RBTree_get_root(tree)) != len) {\
 			return 0;\
 		}\
