@@ -1,42 +1,93 @@
-#include "test.h"
-
+#include <stdio.h>
 #include <string.h>
+#include <assert.h>
 #include <time.h>
+#include "../cstl/ring.h"
+#include "../cstl/set.h"
+#include "rbtree_debug.h"
+#include "heap.h"
+#ifdef MY_MALLOC
+double buf[1024*1024/sizeof(double)];
+Heap heap;
+#define malloc(s)		Heap_alloc(&heap, s)
+#define realloc(p, s)	Heap_realloc(&heap, p, s)
+#define free(p)			Heap_free(&heap, p)
+#endif
+
+
+/* ring */
+CSTL_RING_INTERFACE(IntRing, int)
+CSTL_RING_IMPLEMENT(IntRing, int)
+
+
+
+/* set */
+#ifdef CSTLGEN
+#include "IntSetA.h"
+#include "IntSetD.h"
+#include "IntMSetA.h"
+#include "DoubleSetA.h"
+#include "PtrSetA.h"
+#include "StrSetA.h"
+#include "UIntSetA.h"
+#else
+
+CSTL_SET_INTERFACE(IntSetA, int)
+CSTL_SET_DEBUG_INTERFACE(IntSetA)
+
+CSTL_SET_INTERFACE(IntSetD, int)
+CSTL_SET_DEBUG_INTERFACE(IntSetD)
+
+CSTL_MULTISET_INTERFACE(IntMSetA, int)
+CSTL_SET_DEBUG_INTERFACE(IntMSetA)
+
+CSTL_SET_INTERFACE(DoubleSetA, double)
+CSTL_SET_DEBUG_INTERFACE(DoubleSetA)
+
+CSTL_SET_INTERFACE(PtrSetA, int*)
+CSTL_SET_DEBUG_INTERFACE(PtrSetA)
+
+CSTL_SET_INTERFACE(StrSetA, char*)
+CSTL_SET_DEBUG_INTERFACE(StrSetA)
+
+CSTL_SET_INTERFACE(UIntSetA, unsigned int)
+CSTL_SET_DEBUG_INTERFACE(UIntSetA)
 
 /* int */
-SET_IMPLEMENT(IntSetA, int, SIMPLE_CMP, ASC)
-SET_DEBUG_IMPLEMENT(IntSetA, int, SIMPLE_CMP, ASC, %d, VISUAL)
-static IntSetA *ia;
+CSTL_SET_IMPLEMENT(IntSetA, int, CSTL_LESS)
+CSTL_SET_DEBUG_IMPLEMENT(IntSetA, int, CSTL_LESS, %d, VISUAL)
 
-SET_IMPLEMENT(IntSetD, int, SIMPLE_CMP, DESC)
-SET_DEBUG_IMPLEMENT(IntSetD, int, SIMPLE_CMP, DESC, %d, VISUAL)
-static IntSetD *id;
+CSTL_SET_IMPLEMENT(IntSetD, int, CSTL_GREATER)
+CSTL_SET_DEBUG_IMPLEMENT(IntSetD, int, CSTL_GREATER, %d, VISUAL)
 
-MULTISET_IMPLEMENT(IntMSetA, int, SIMPLE_CMP, ASC)
-SET_DEBUG_IMPLEMENT(IntMSetA, int, SIMPLE_CMP, ASC, %d, VISUAL)
-static IntMSetA *ima;
+CSTL_MULTISET_IMPLEMENT(IntMSetA, int, CSTL_LESS)
+CSTL_SET_DEBUG_IMPLEMENT(IntMSetA, int, CSTL_LESS, %d, VISUAL)
 
 
 /* double */
-SET_IMPLEMENT(DoubleSetA, double, SIMPLE_CMP, ASC)
-SET_DEBUG_IMPLEMENT(DoubleSetA, double, SIMPLE_CMP, ASC, %g, VISUAL)
-static DoubleSetA *da;
+CSTL_SET_IMPLEMENT(DoubleSetA, double, CSTL_LESS)
+CSTL_SET_DEBUG_IMPLEMENT(DoubleSetA, double, CSTL_LESS, %g, VISUAL)
 
 
 /* ptr */
-SET_IMPLEMENT(PtrSetA, int*, SIMPLE_CMP, ASC)
-SET_DEBUG_IMPLEMENT(PtrSetA, int*, SIMPLE_CMP, ASC, %p, VISUAL)
-static PtrSetA *pa;
+CSTL_SET_IMPLEMENT(PtrSetA, int*, CSTL_LESS)
+CSTL_SET_DEBUG_IMPLEMENT(PtrSetA, int*, CSTL_LESS, %p, VISUAL)
 
 
 /* str */
-SET_IMPLEMENT(StrSetA, char*, strcmp, ASC)
-SET_DEBUG_IMPLEMENT(StrSetA, char*, strcmp, ASC, %s, VISUAL)
-static StrSetA *sa;
+CSTL_SET_IMPLEMENT(StrSetA, char*, strcmp)
+CSTL_SET_DEBUG_IMPLEMENT(StrSetA, char*, strcmp, %s, VISUAL)
 
 /* unsigned int */
-SET_IMPLEMENT(UIntSetA, unsigned int, SIMPLE_CMP, ASC)
-SET_DEBUG_IMPLEMENT(UIntSetA, unsigned int, SIMPLE_CMP, ASC, %d, VISUAL)
+CSTL_SET_IMPLEMENT(UIntSetA, unsigned int, CSTL_LESS)
+CSTL_SET_DEBUG_IMPLEMENT(UIntSetA, unsigned int, CSTL_LESS, %d, VISUAL)
+#endif
+static IntSetA *ia;
+static IntSetD *id;
+static IntMSetA *ima;
+static DoubleSetA *da;
+static PtrSetA *pa;
+static StrSetA *sa;
 static UIntSetA *uia;
 
 
@@ -52,17 +103,17 @@ void set_init_hoge(void)
 {
 	int i;
 	int tmp;
-	IntDeque *q = IntDeque_new(SIZE/2);
+	IntRing *q = IntRing_new(SIZE/2);
 	srand(time(0));
 /*    printf("hoge data\n");*/
 	for (i = 0; i < SIZE/2; i++) {
-		IntDeque_push_back(q, i);
+		IntRing_push_back(q, i);
 	}
 	for (i = 0; i < SIZE; i++) {
-		if (!IntDeque_empty(q)) {
-			int rnd = rand() % IntDeque_size(q);
-			tmp = *IntDeque_at(q, rnd);
-			IntDeque_erase(q, rnd, 1);
+		if (!IntRing_empty(q)) {
+			int rnd = rand() % IntRing_size(q);
+			tmp = *IntRing_at(q, rnd);
+			IntRing_erase(q, rnd, 1);
 		} else {
 			tmp = i/2;
 		}
@@ -74,7 +125,8 @@ void set_init_hoge(void)
 /*        printf("%4d: int[%3d], double[%5g], ptr[%p], str[%s]\n",*/
 /*                i, hoge_int[i], hoge_double[i], hoge_ptr[i], hoge_str[i]);*/
 	}
-	IntDeque_delete(q);
+	HEAP_DUMP_OVERFLOW(&heap);
+	IntRing_delete(q);
 }
 
 void SetTest_test_1_1(void)
@@ -96,6 +148,7 @@ void SetTest_test_1_1(void)
 	/* insert */
 	for (i = 0; i < SIZE; i++) {
 		pos[i] = IntSetA_insert(ia, hoge_int[i], &success[i]);
+		assert(IntSetA_verify(ia));
 		assert(pos[i]);
 		if (i < SIZE/2) {
 			assert(success[i]);
@@ -136,6 +189,7 @@ void SetTest_test_1_1(void)
 		if (pos[i] && success[i]) {
 			IntSetAIterator itr = IntSetA_next(pos[i]);
 			assert(itr == IntSetA_erase(ia, pos[i]));
+			assert(IntSetA_verify(ia));
 			count--;
 		}
 	}
@@ -149,6 +203,7 @@ void SetTest_test_1_1(void)
 	}
 	assert(IntSetA_size(ia) == SIZE/2);
 	assert(IntSetA_find(ia, SIZE/2 -2) == IntSetA_erase_range(ia, IntSetA_find(ia, 2), IntSetA_find(ia, SIZE/2 -2)));
+	assert(IntSetA_verify(ia));
 	assert(IntSetA_size(ia) == 4);
 	assert(IntSetA_end(ia) == IntSetA_erase_range(ia, IntSetA_begin(ia), IntSetA_end(ia)));
 	assert(IntSetA_size(ia) == 0);
@@ -169,6 +224,7 @@ void SetTest_test_1_1(void)
 	assert(IntSetA_size(ia) == SIZE/2);
 	for (i = 0; i < SIZE/2; i++) {
 		assert(IntSetA_erase_key(ia, hoge_int[i]) == 1);
+		assert(IntSetA_verify(ia));
 	}
 	assert(IntSetA_size(ia) == 0);
 	/* 大量にinsert */
@@ -208,17 +264,34 @@ void SetTest_test_1_1(void)
 
 	IntSetA_swap(ia, x);
 
+	assert(IntSetA_verify(ia));
+	assert(IntSetA_verify(x));
 /*    IntSetA_print(ia);*/
 /*    IntSetA_print(x);*/
 	assert(IntSetA_size(x) == SIZE/2);
 	assert(IntSetA_size(ia) == sizeof b / sizeof b[0]);
-	/* assign */
-	assert(IntSetA_assign(x, IntSetA_begin(ia), IntSetA_end(ia)));
-	assert(IntSetA_size(ia) == sizeof b / sizeof b[0]);
-	assert(IntSetA_size(x)  == sizeof b / sizeof b[0]);
-	IntSetA_print(ia);
-	IntSetA_print(x);
+	/* insert_range */
+	count = IntSetA_size(x);
+	IntSetA_insert(x, b[0], 0);
+	IntSetA_insert(ia, hoge_int[0], 0);
+/*    IntSetA_print(ia);*/
+/*    IntSetA_print(x);*/
+	assert(IntSetA_insert_range(x, IntSetA_begin(ia), IntSetA_end(ia)));
 
+/*    IntSetA_print(ia);*/
+/*    IntSetA_print(x);*/
+	assert(IntSetA_size(ia) == sizeof b / sizeof b[0] + 1);
+	assert(IntSetA_size(x)  == count + sizeof b / sizeof b[0]);
+	assert(IntSetA_verify(ia));
+	assert(IntSetA_verify(x));
+
+	assert(IntSetA_insert_range(x, IntSetA_begin(ia), IntSetA_end(ia)));
+	assert(IntSetA_size(ia) == sizeof b / sizeof b[0] + 1);
+	assert(IntSetA_size(x)  == count + sizeof b / sizeof b[0]);
+	assert(IntSetA_verify(ia));
+	assert(IntSetA_verify(x));
+
+	HEAP_DUMP_OVERFLOW(&heap);
 	IntSetA_delete(ia);
 	IntSetA_delete(x);
 }
@@ -323,6 +396,7 @@ void SetTest_test_1_2(void)
 	assert(IntSetD_size(id) == count);
 	printf("count: %d\n", count);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	IntSetD_delete(id);
 }
 
@@ -332,6 +406,8 @@ void SetTest_test_1_3(void)
 	size_t count = 0;
 	IntMSetAIterator pos[SIZE];
 	IntMSetAIterator p;
+	IntMSetA *x;
+	int b[] = {100, 109, 101, 108, 102, 103, 104, 104, 105, 106, 107, 100, 101, 101, 108, 109};
 	int flag[SIZE/2] = {0};
 	printf("***** test_1_3 *****\n");
 	ima = IntMSetA_new();
@@ -445,7 +521,37 @@ void SetTest_test_1_3(void)
 	assert(IntMSetA_size(ima) == count);
 	printf("count: %d\n", count);
 
+	IntMSetA_clear(ima);
+
+	/* insert_range */
+	for (i = 0; i < SIZE/2; i++) {
+		pos[i] = IntMSetA_insert(ima, hoge_int[i]);
+		assert(pos[i]);
+	}
+	assert(IntMSetA_size(ima) == SIZE/2);
+	x = IntMSetA_new();
+	for (i = 0; i < sizeof b / sizeof b[0]; i++) {
+		pos[i] = IntMSetA_insert(x, b[i]);
+		assert(pos[i]);
+	}
+	for (i = 0; i < SIZE/2; i++) {
+		pos[i] = IntMSetA_insert(x, hoge_int[i]);
+		assert(pos[i]);
+	}
+	assert(IntMSetA_size(x) == sizeof b / sizeof b[0] + SIZE/2);
+/*    IntMSetA_print(ima);*/
+/*    IntMSetA_print(x);*/
+	assert(IntMSetA_insert_range(x, IntMSetA_begin(ima), IntMSetA_end(ima)));
+	assert(IntMSetA_verify(ima));
+	assert(IntMSetA_verify(x));
+/*    IntMSetA_print(ima);*/
+/*    IntMSetA_print(x);*/
+	assert(IntMSetA_size(x) == sizeof b / sizeof b[0] + SIZE);
+
+
+	HEAP_DUMP_OVERFLOW(&heap);
 	IntMSetA_delete(ima);
+	IntMSetA_delete(x);
 }
 
 void SetTest_test_2_1(void)
@@ -558,6 +664,7 @@ void SetTest_test_2_1(void)
 	DoubleSetA_clear(da);
 	assert(DoubleSetA_size(da) == 0);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	DoubleSetA_delete(da);
 }
 
@@ -673,6 +780,7 @@ void SetTest_test_3_1(void)
 	PtrSetA_clear(pa);
 	assert(PtrSetA_size(pa) == 0);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	PtrSetA_delete(pa);
 }
 
@@ -789,6 +897,7 @@ void SetTest_test_4_1(void)
 	StrSetA_clear(sa);
 	assert(StrSetA_size(sa) == 0);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	StrSetA_delete(sa);
 }
 
@@ -908,6 +1017,7 @@ void SetTest_test_5_1(void)
 	UIntSetA_clear(uia);
 	assert(UIntSetA_size(uia) == 0);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	UIntSetA_delete(uia);
 }
 
@@ -939,3 +1049,15 @@ void SetTest_run(void)
 
 }
 
+
+int main(void)
+{
+#ifdef MY_MALLOC
+	Heap_init(&heap, buf, sizeof buf, sizeof buf[0]);
+#endif
+	SetTest_run();
+#ifdef MY_MALLOC
+	HEAP_DUMP_LEAK(&heap, 0);
+#endif
+	return 0;
+}
