@@ -1,4 +1,5 @@
 #include "../cstl/string.h"
+#include "../cstl/algorithm.h"
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
@@ -6,14 +7,19 @@
 
 #include "heap.h"
 #ifdef MY_MALLOC
-extern Heap heap;
+double buf[1024*1024/sizeof(double)];
+Heap heap;
 #define malloc(s)		Heap_alloc(&heap, s)
 #define realloc(p, s)	Heap_realloc(&heap, p, s)
 #define free(p)			Heap_free(&heap, p)
 #endif
 
+#ifdef CSTLGEN
+#include "String.h"
+#else
 CSTL_STRING_INTERFACE(String, char)
 CSTL_STRING_IMPLEMENT(String, char)
+#endif
 
 #define SIZE	16
 
@@ -45,6 +51,7 @@ void StringTest_test_1_1(void)
 	assert(String_empty(x));
 	assert(String_size(x) == 0);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	String_delete(x);
 }
 
@@ -133,6 +140,7 @@ void StringTest_test_1_2(void)
 	assert(String_size(y) == 7);
 	assert(String_size(x) == 8);
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	String_delete(x);
 	String_delete(y);
 }
@@ -234,8 +242,29 @@ void StringTest_test_1_3(void)
 	assert(strcmp("gijkhabcdeflmopqn", String_c_str(x)) == 0);
 	String_insert(x, 1, String_c_str(x), CSTL_NPOS);
 	assert(String_size(x) == 34);
-	printf("%s\n", String_c_str(x));
 	assert(strcmp("ggijkhabcdeflmopqnijkhabcdeflmopqn", String_c_str(x)) == 0);
+	String_erase(x, 1, 17);
+	assert(String_size(x) == 17);
+	assert(strcmp("gijkhabcdeflmopqn", String_c_str(x)) == 0);
+
+	String_insert(x, 0, String_c_str(x) + 1, 4);
+	assert(String_size(x) == 21);
+	assert(strcmp("ijkhgijkhabcdeflmopqn", String_c_str(x)) == 0);
+	String_erase(x, 0, 4);
+	assert(String_size(x) == 17);
+	assert(strcmp("gijkhabcdeflmopqn", String_c_str(x)) == 0);
+
+	String_insert(x, 3, String_c_str(x) + 1, 4);
+	assert(String_size(x) == 21);
+	assert(strcmp("gijijkhkhabcdeflmopqn", String_c_str(x)) == 0);
+	String_erase(x, 3, 4);
+	assert(String_size(x) == 17);
+	assert(strcmp("gijkhabcdeflmopqn", String_c_str(x)) == 0);
+
+	String_insert(x, 5, String_c_str(x) + 1, 4);
+	assert(String_size(x) == 21);
+	assert(strcmp("gijkhijkhabcdeflmopqn", String_c_str(x)) == 0);
+
 	String_clear(x);
 	assert(String_size(x) == 0);
 	/* insert_c */
@@ -280,6 +309,17 @@ void StringTest_test_1_3(void)
 	String_delete(x);
 }
 
+int cmp(const void *x, const void *y)
+{
+	if (*(char*)x < *(char*)y) {
+		return -1;
+	} else if (*(char*)x > *(char*)y) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
 void StringTest_test_1_4(void)
 {
 	String *x;
@@ -308,16 +348,46 @@ void StringTest_test_1_4(void)
 	String_replace(x, 0, 3, "hogehoge", 4);
 	assert(String_size(x) == 27);
 	assert(strcmp("hogeDEFGHIjklmnopqrstuvwXYZ", String_c_str(x)) == 0);
+
 	String_replace(x, 3, 4, String_c_str(x), 4);
 	assert(String_size(x) == 27);
 	printf("%s\n", String_c_str(x));
 	assert(strcmp("hoghogeGHIjklmnopqrstuvwXYZ", String_c_str(x)) == 0);
+
+	String_replace(x, 2, 4, String_c_str(x), 3);
+	assert(String_size(x) == 26);
+	printf("%s\n", String_c_str(x));
+	assert(strcmp("hohogeGHIjklmnopqrstuvwXYZ", String_c_str(x)) == 0);
+
+	String_replace(x, 4, 4, String_c_str(x) + 6, 3);
+	assert(String_size(x) == 25);
+	printf("%s\n", String_c_str(x));
+	assert(strcmp("hohoGHIIjklmnopqrstuvwXYZ", String_c_str(x)) == 0);
+
+	String_erase(x, 0, 8);
+	assert(String_size(x) == 17);
+	printf("%s\n", String_c_str(x));
+	assert(strcmp("jklmnopqrstuvwXYZ", String_c_str(x)) == 0);
+
+	String_replace(x, 2, 4, String_c_str(x), 5);
+	assert(String_size(x) == 18);
+	printf("%s\n", String_c_str(x));
+	assert(strcmp("jkjklmnpqrstuvwXYZ", String_c_str(x)) == 0);
+
+	String_replace(x, 4, 4, String_c_str(x) + 6, 5);
+	assert(String_size(x) == 19);
+	printf("%s\n", String_c_str(x));
+	assert(strcmp("jkjknpqrsqrstuvwXYZ", String_c_str(x)) == 0);
+
+
 	String_replace(x, 0, CSTL_NPOS, "", CSTL_NPOS);
 	assert(String_size(x) == 0);
 	assert(strcmp("", String_c_str(x)) == 0);
+
 	String_replace(x, 0, CSTL_NPOS, "abc", CSTL_NPOS);
 	assert(String_size(x) == 3);
 	assert(strcmp("abc", String_c_str(x)) == 0);
+
 	/* replace_c */
 	String_assign(x, "abcdefghijklmnopqrstuvwxyz", CSTL_NPOS);
 	assert(String_size(x) == 26);
@@ -347,6 +417,12 @@ void StringTest_test_1_4(void)
 	String_replace_c(x, 0, CSTL_NPOS, 10, '0');
 	assert(String_size(x) == 10);
 	assert(strcmp("0000000000", String_c_str(x)) == 0);
+	/* sort */
+	String_assign(x, "asortingexample", CSTL_NPOS);
+	printf("String_sort\n");
+	printf("%s\n", String_c_str(x));
+	String_sort(x, 0, String_size(x), cmp);
+	printf("%s\n", String_c_str(x));
 
 #if 0
 	string s = "abcdefghijklmnopqrstuvwxyz";
@@ -354,6 +430,7 @@ void StringTest_test_1_4(void)
 	printf("%s\n", s.c_str());
 #endif
 
+	HEAP_DUMP_OVERFLOW(&heap);
 	String_delete(x);
 }
 
@@ -369,7 +446,7 @@ void print_find_test(const char *cstr, string &t, size_t i, size_t j, size_t k,
 	}
 }
 
-void find_test(String *x, const string &s, size_t i, size_t j, size_t k, char *cstr)
+void find_test(String *x, const string &s, size_t i, size_t j, size_t k, const char *cstr)
 {
 	string t;
 	size_t idx1, idx2;
@@ -510,17 +587,15 @@ void StringTest_run(void)
 	StringTest_test_1_5();
 }
 
-double buf[1024*1024/sizeof(double)];
-Heap heap;
 
 int main(void)
 {
 #ifdef MY_MALLOC
-	Heap_init(&heap, buf, sizeof buf);
+	Heap_init(&heap, buf, sizeof buf, sizeof buf[0]);
 #endif
 	StringTest_run();
 #ifdef MY_MALLOC
-	DUMP_MEMORY_LEAK(&heap, 0);
+	HEAP_DUMP_LEAK(&heap, 0);
 #endif
 	return 0;
 }
